/* help.c
 
   Routines for the Lorien Help system.
   Written by Jerry A. Bolton, Jr.

*/


#include <stdio.h>
#include <sys/types.h>
#ifdef POSIX
#include <sys/select.h>
#endif
#include <sys/time.h>
#include <sys/resource.h>

#include <string.h>
#include <ctype.h>
#include "lorien.h"

void 
strupcase(s)
    char *s;
{
    int i;

    for (i = 0; s[i]; i++)
	if (isalpha(s[i]))
	    s[i] = toupper(s[i]);
}

#define EXACT 1
#define SUB   2
#define REGEX 4 /* not yet implemented */

/* don't pass strings to strcontains which are any larger than BUFSIZE */

char *
strcontains(string,target,mode)
    char *string;
    char *target;
    int   mode;
{
    char s[BUFSIZE];
    char t[BUFSIZE];

    strcpy(s,string);
    strcpy(t,target);

    strupcase(s);
    strupcase(t);

    fflush(stdout);

    if (mode == SUB) return ((char *)strstr(s,t));
    return (!strcmp(s,t) ? string : (char *) 0);

}

void
showhelp(pplayer,buf)
    struct splayer *pplayer;
    char *buf;
{
    FILE *fHelp = fopen(HELPFILE,"r");
    int i,level;
    char *inbuf;
    char ibuf[BUFSIZE];
    char tagbuf[BUFSIZE];
    char outbuf[OBUFSIZE];
    char target[BUFSIZE];

    sendtoplayer(pplayer,">> Lorien Help System v1.1a\r\n");

    if (!fHelp) {
	sendtoplayer(pplayer,">> Unable to open help file.\r\n");
	return;
    }

    while (isspace(*buf)) buf++;

    if (buf[0] == '/' || buf[0] == '.' || buf[0] == ',')
	buf++;

    if (strlen(buf))
	strcpy(target,buf);
    else 
	strcpy(target,"section");

    rewind(fHelp);
    while (fgets(ibuf,BUFSIZE,fHelp) != (char *) 0) {
	char *tmp;
	inbuf = ibuf;

	    /* remove newline */

	tmp = strstr(inbuf,"\n");
	if (tmp) *tmp = (char) 0; 

	    /* check security level */

	if (isdigit(*inbuf)) {
	    level = atoi(inbuf);
	    if (level > pplayer->seclevel)
		continue;
	    while (isdigit(*inbuf)) inbuf++;
	}

	    /* discard comments. */

	if (*inbuf == '#')
	    continue;

	switch(*inbuf) {
	case 'M': tmp = strstr(inbuf,"|") + 1;
	    sprintf(outbuf,"%s\r\n",tmp);
	    sendtoplayer(pplayer,outbuf);
	    break;
	case 'T': inbuf++;
	    do {
		i = strcspn(inbuf,",|");
		if (i) {
		    strncpy(tagbuf,inbuf,i);
		    tagbuf[i] = (char) 0;
		    if (strcontains(tagbuf,target,EXACT)) {
			tmp = strchr(inbuf,'|') + 1;
			sprintf(outbuf,"%s\r\n",tmp);
			sendtoplayer(pplayer,outbuf);
			break;
		    }
		    else {
			inbuf+= i + 1;
		    }
		}
	    } while (i);
	    break;
	case 'B': tmp = strstr(inbuf,"|") + 1;
	    sprintf(outbuf,"%s\r\n",tmp);
	    sendtoplayer(pplayer,outbuf);
	    break;
	}
    }

    sendtoplayer(pplayer,">> End of help.\r\n");
    fclose(fHelp);
}
